C File <igram.f> containing the Inverse Gram determinants.
C The functions are given with the following convention:
C IGRAM`N"=1/DET(2p1.p1 2p1.p2 , ..., 2p1.pN \\ 2p2.p1 .\\...\\. 2pN.pN)
C This file is part of aITALC
C Copyright (C) 2003,2004 Alejandro Lorca <alejandro.lorca@desy.de>
C ----------------------------------------------------------------------

      DOUBLE PRECISION FUNCTION IGRAM1(PP1)
      IMPLICIT NONE
      DOUBLE PRECISION PP1
      DOUBLE PRECISION GRAM1

      IF (PP1.NE.0D0) THEN

        GRAM1=2*PP1

        IGRAM1=1/GRAM1

      ELSE
        PRINT *,'IGRAM1: ZERO GRAM DETERMINANT!, SINGULARITY, STOP'
        STOP
      ENDIF

      RETURN
      END
C ----------------------------------------------------------------------

      DOUBLE PRECISION FUNCTION IGRAM2(PP1,QQ3,PP2)
      IMPLICIT NONE
      INTEGER nmax
      PARAMETER (nmax=6)
      DOUBLE PRECISION PP1,QQ3,PP2
      DOUBLE PRECISION GRAM2,gramarray(nmax),order(nmax)
      INTEGER i1,i2,nn(nmax)

C In case of the 2 same arguments, then they match at beginning or end


      gramarray(1)=+2*PP1*PP2
      gramarray(2)=+2*PP1*QQ3
      gramarray(3)=-PP1*PP1
      gramarray(4)=+2*PP2*QQ3
      gramarray(5)=-PP2*PP2
      gramarray(6)=-QQ3*QQ3


      DO i1=1,nmax
        order(i1)=0d0
        nn(i1)=0
        DO i2=1,nmax
          IF(DABS(gramarray(i2)).GT.DABS(order(i1))) THEN
            order(i1)=gramarray(i2)
            nn(i1)=i2
          ENDIF
        ENDDO
        gramarray(nn(i1))=0d0
      ENDDO

      GRAM2=order(1)
      DO i1=2,nmax
        GRAM2=GRAM2+order(i1)
      ENDDO

      IF (GRAM2.EQ.0D0) PRINT *,'IGRAM2: GRAM2 = 0'



c      IF (PP1.EQ.QQ3) THEN
c        IF ((PP2.NE.4*PP1).AND.(PP2.NE.0D0)) THEN
c          GRAM2= + PP2 * ( +4*PP1 - PP2)
c        ELSE
c          GOTO 20
c        ENDIF
c      ELSEIF (QQ3.EQ.PP2) THEN
c        IF ((PP1.NE.4*PP2).AND.(PP2.NE.0D0)) THEN
c          GRAM2= + PP1 * ( +4*PP2 - PP1)
c        ELSE
c          GOTO 20
c        ENDIF
c      ELSEIF ((PP1.NE.QQ3).AND.(PP1.NE.PP2).AND.(QQ3.NE.PP2)) THEN
c        GRAM2=+(
c     &    +PP1*(2*PP2+2*QQ3-PP1)
c     &    +PP2*(2*QQ3-PP2)
c     &    -QQ3**2
c     &    )
c      ELSE
c 20     PRINT *,'IGRAM2: ZERO GRAM DETERMINANT!, SINGULARITY, STOP'
c        STOP
c      ENDIF

      IGRAM2=2/GRAM2

      RETURN
      END
C ----------------------------------------------------------------------

      DOUBLE PRECISION FUNCTION IGRAM3(PP1,QQ3,QQ1,PP3,PP2,QQ2)
      IMPLICIT NONE
      INTEGER nmax
      parameter (nmax=22)
      DOUBLE PRECISION PP1,QQ3,QQ1,PP3,PP2,QQ2
      DOUBLE PRECISION GRAM3,gramarray(nmax),order(nmax)
      INTEGER i1,i2,nn(nmax)

C I know next is MAYBE not the most efficient, but just in the meanwhile

c      GRAM3=     +PP2*QQ2*(+PP1+PP3+QQ3+QQ1-QQ2-PP2)
c      GRAM3=GRAM3+PP2*((QQ1-QQ3)*(PP1-PP3))
c      GRAM3=GRAM3+QQ2*((QQ1-PP3)*(PP1-QQ3))
c      GRAM3=GRAM3+(PP3*QQ3-PP1*QQ1)*(+PP1+QQ1-QQ3-PP3)

c      GRAM3= GRAM3/2

      gramarray(1)=-PP1*PP2*QQ3
      gramarray(2)=+PP1*PP2*QQ2
      gramarray(3)=+PP1*PP2*QQ1
      gramarray(4)=+PP1*PP3*QQ3
      gramarray(5)=-PP1*PP3*QQ2
      gramarray(6)=+PP1*PP3*QQ1
      gramarray(7)=+PP1*QQ3*QQ1
      gramarray(8)=+PP1*QQ2*QQ1
      gramarray(9)=-PP1*QQ1*QQ1
      gramarray(10)=-PP1*PP1*QQ1
      gramarray(11)=+PP2*PP3*QQ3
      gramarray(12)=+PP2*PP3*QQ2
      gramarray(13)=-PP2*PP3*QQ1
      gramarray(14)=+PP2*QQ3*QQ2
      gramarray(15)=+PP2*QQ2*QQ1
      gramarray(16)=-PP2*QQ2*QQ2
      gramarray(17)=-PP2*PP2*QQ2
      gramarray(18)=+PP3*QQ3*QQ2
      gramarray(19)=+PP3*QQ3*QQ1
      gramarray(20)=-PP3*QQ3*QQ3
      gramarray(21)=-PP3*PP3*QQ3
      gramarray(22)=-QQ3*QQ2*QQ1

      DO i1=1,nmax
        order(i1)=0d0
        nn(i1)=0
        DO i2=1,nmax
          IF(DABS(gramarray(i2)).GT.DABS(order(i1))) THEN
            order(i1)=gramarray(i2)
            nn(i1)=i2
          ENDIF
        ENDDO
        gramarray(nn(i1))=0d0
      ENDDO

      GRAM3=order(1)
      DO i1=2,nmax
        GRAM3=GRAM3+order(i1)
      ENDDO

      IF (GRAM3.EQ.0D0) PRINT *,'IGRAM3: GRAM3 = 0'

      IGRAM3=2/GRAM3

      RETURN
      END
C ----------------------------------------------------------------------
CC $Id: igram.f,v 1.2 2004/10/07 15:05:12 alorca Exp $
